# -*- coding: utf=8 -*-
import ast
from datetime import datetime, timedelta
import requests


class Autenticacao:
    instances = []

    def __init__(self):
        self.token = None
        self.data_hora_limite = None

    def set_token(self, result):
        self.token = result
        data_hora = datetime.now()
        self.data_hora_limite = data_hora + timedelta(minutes=100)
        self.instances.append(self)

    def get_token(self):
        """verifica se tem token e se ele está denro do prazo estipulado"""
        data_hora = datetime.now()
        if self.data_hora_limite > data_hora:
            return self.token
        return False

    def autentica(self, url, cert, client_id, client_secret):
        """Autenticacao de homologacao"""
        url +="/oauth/token?grant_type=client_credentials"

        payload = 'client_id={}&client_secret={}'.format(client_id, client_secret)
        headers = {
            'Content-Type': 'application/x-www-form-urlencoded'
        }
        response = requests.request("POST", url, headers=headers, data=payload, cert=cert)

        return response


def get_autenticacao_token(url, cert, client_id, client_secret):
    """Realiza a autenticação através da classe Autenticacao """
    autenticacoes = Autenticacao.instances
    """verifica se existe token válido em memória"""
    if autenticacoes and autenticacoes[0].get_token():
        '''o token sempre estará na posição zero'''
        result = autenticacoes[0].get_token()
        result = ast.literal_eval(result.text)
        return result
    else:
        """Cria um token válido no santander"""
        autenticacao_obj = Autenticacao()
        autenticacao = autenticacao_obj.autentica(url, cert, client_id, client_secret)

        if autenticacao.status_code == 200:
            result = ast.literal_eval(autenticacao.text)
            if 'access_token' in result:
                if autenticacoes:
                    '''atualiza o primeiro registro com um token válido'''
                    autenticacoes[0].set_token(autenticacao)
                else:
                    autenticacao_obj.set_token(autenticacao)
                return result
        """pode retornar erro interno devido a instabilidades do santander"""
        return {"status_code": 500, "mensagem": "Erro interno"}




# python3 setup.py sdist upload -r testpypi
# pip install -i https://test.pypi.org/simple/ lins_pix==0.0.14
# requests~=2.25.1
# urllib3~=1.26.4
# PyJWT~=1.7.1
# websocket-client~=0.53.0


'''
Reúne endpoints destinados a lidar com gerenciamento de cobranças imediatas
'''
import requests

from ..santander.autenticacao import get_autenticacao_token


def checa_validade_dados(body):
    '''
        Verifica validade dos dados e caso positivo retorna os dados em um dicionario
    '''
    try:
        valor = body['valor']["original"]
        try:
            tipo_pessoa = "cnpj"
            doc_pessoa = body['devedor']["cnpj"]
        except:
            doc_pessoa = body['devedor']["cpf"]
            tipo_pessoa = "cpf"

        nome = body['devedor']["nome"]
        expiracao = body['calendario']['expiracao']
        info_adicionais = body['infoAdicionais']
        chave = body['chave']
        solicitacao_pagador = body['solicitacaoPagador']

        data = {
            "calendario": {
                "expiracao": expiracao
            },
            "devedor": {
                tipo_pessoa: doc_pessoa,
                "nome": nome
            },
            "valor": {
                "original": valor
            },
            "chave": chave,
            "solicitacaoPagador": solicitacao_pagador,
            "infoAdicionais": info_adicionais,
        }
        data = str(data).replace("'", '"')
        return data
    except:

        return False


class Cob:
    def __init__(self):
        self.x = 1

    def criar_cobranca_put(self, txid, body, url, cert, client_id, client_secret):

        '''
        TXID entre 27 e 35 caracteres
        Criar cobrança imediata.
        Endpoint para criar uma cobrança imediata.
        PUT - /cob/{txid}

        Status code tratados:
        201 (sucesso) - Cobrança imediata criada.
        400 (erro) - Requisição com formato inválido.
        403 (erro) - Requisição de participante autenticado que viola alguma regra de autorização.
        404 (erro) - Recurso solicitado não foi encontrado.
        503 (erro) - Serviço não está disponível no momento. Serviço solicitado pode estar em manutenção ou fora da janela de funcionamento.
        '''
        data = checa_validade_dados(body)
        if not data or len(txid) < 27:
            result = {
                "title": "Cobrança inválida.",
                "status": 400,
                "detail": "A requisição que busca alterar ou criar uma cobrança para pagamento imediato não respeita o "
                          "schema ou está semanticamente errada."
            }
            return result, 400

        autenticacao = get_autenticacao_token(url, cert, client_id, client_secret)
        if not 'access_token' in autenticacao:
            result = {
                "title": "Erro ao autenticar",
                "status": 400,
                "detail:": "Erro na autenticação na api do Santader."
            }
            return result, 400
        access_token = autenticacao['access_token']

        headers = {
            'Authorization': 'Bearer {}'.format(access_token),
            'Content-Type': 'application/json'
        }
        url += "/api/v1/cob/{}".format(str(txid))
        result = requests.request("PUT", url, headers=headers, data=data, cert=cert)

        return result.text, result.status_code

    def consultar_cobranca_get(self, txid, url, cert, client_id, client_secret):
        '''
        Consultar cobrança imediata.
        Endpoint para consultar uma cobrança através de um determinado txid.
        GET - /cob/{txid}

        Status code tratados:
        200 (sucesso) - Dados da cobrança imediata.
        400 (erro) - Erro na autenticação na api do Santader
        403 (erro) - Requisição de participante autenticado que viola alguma regra de autorização.
        404 (erro) - Recurso solicitado não foi encontrado.
        503 (erro) - Serviço não está disponível no momento. Serviço solicitado pode estar em manutenção ou
         fora da janela de funcionamento.
        '''
        if len(txid) < 27:
            result = {
                "title": "Cobrança inválida.",
                "status": 400,
                "detail": "Quantidade de caracteres do TXID inválido."
            }
            return result, 400
        autenticacao = get_autenticacao_token(url, cert, client_id, client_secret)
        if not 'access_token' in autenticacao:
            result = {
                "title": "Erro ao autenticar",
                "status": 400,
                "detail:": "Erro na autenticação na api do Santader."
            }
            return result, 400
        access_token = autenticacao['access_token']

        url += "/api/v1/cob/{}".format(str(txid))
        headers = {
            'Authorization': 'Bearer {}'.format(access_token),
            'Content-Type': 'application/json'
        }
        result = requests.request("GET", url, headers=headers, data=None, cert=cert)

        return result.text, result.status_code

    def consultar_lista_cobranca_get(self, inicio, fim, body, url, cert, client_id, client_secret):

        """Não definido pela área de negócio"""
        '''
        Todas as entradas são no formato de string e tem como campo obrigatório inicio e fim.
        O Body deve trazer um dicionario com as seguintes opções:
        cpf ou cnpj, locationPresent, status, paginaAtual, itensPorPagina

        Consultar lista de cobranças imediatas.
        Endpoint para consultar cobranças imediatas através de parâmetros como início, fim, cpf, cnpj e status.
        GET - /cob

        Status code tratados:
        200 (sucesso) - Lista de cobranças imediatas.
        403 (erro) - Requisição de participante autenticado que viola alguma regra de autorização.
        503 (erro) - Serviço não está disponível no momento. Serviço solicitado pode estar em manutenção ou fora da 
        janela de funcionamento.
        '''
        autenticacao = get_autenticacao_token(url, cert, client_id, client_secret)
        if not 'access_token' in autenticacao:
            result = {
                "title": "Erro ao autenticar",
                "status": 400,
                "detail:": "Erro na autenticação na api do Santader."
            }
            return result, 400
        access_token = autenticacao['access_token']
        headers = {
            'Authorization': 'Bearer {}'.format(access_token),
            'Content-Type': 'application/json'
        }

        url += "/api/v1/pix/" + '?inicio=' + inicio + '&fim=' + fim

        if body:
            for item in body:
                url += '&{}={}'.format(item, body[item])

        result = requests.request("GET", url, headers=headers, data=None, cert=cert)

        return result.text, result.status_code




# import json
# import qrcode
# import base64
#
#
# class Qrcode:
#
#     def __init__(self, data):
#         self.data = data
#
#     def to_base64(self):
#         # b64 = base64.b64encode(urlopen(self.data).read()).decode('utf-8')
#         b64 = base64.b64encode(bytes(self.data, 'utf-8'))
#
#         return b64
#
#     def to_imagem(self, path='', name='qrcode.png'):
#         qr = qrcode.QRCode(
#             version=2,
#             error_correction=qrcode.constants.ERROR_CORRECT_L,
#             box_size=10,
#             border=4,
#         )
#         qr.add_data(self.data)
#         qr.make(fit=True)
#
#         img = qr.make_image(fill_color="black", back_color="white")
#         img.save(path + name)
#
#         return img



# python3 setup.py sdist upload -r testpypi
# pip install -i https://test.pypi.org/simple/ lins_pix==0.0.14

from distutils.core import setup
from setuptools import find_packages


def get_version():
    return open('version.txt', 'r').read().strip()


# def get_requirements():
#     return [i.strip() for i in open('requirements-pix.txt').readlines()]


setup(
    name='lins_pix',
    description='Pacote para gerenciar transacoes PIX',
    version='1.0.0.29',
    packages=find_packages(),
    install_requires=[
        'qrcode==6.1',
        'requests==2.25.1',
        'PyJWT==1.7.1',



    ],  # get_requirements(),
    url='https://bitbucket.org/grupolinsferrao/pypck-pix/',
    author='Gustavo Schaedler',
    author_email='gustavopoa@gmail.com',
    license='MIT',
)



# import unittest
# import secrets
# import string
#
# #testes python -m unittest
# from psp.santander.cob import Cob


# cert = None
#
# class ApiSantanderTest(unittest.TestCase):
#
#     def setUp(self):
#         self.txid_valido = ''.join(secrets.choice(string.ascii_uppercase + string.digits) for i in range(27))
#         self.txid_valido1 = ''.join(secrets.choice(string.ascii_uppercase + string.digits) for i in range(27))
#         self.txid_inexistente = ''.join(secrets.choice(string.ascii_uppercase + string.digits) for i in range(27))
#         self.body = {
#                     'calendario': {
#                         'expiracao': '300'
#                     },
#                     'devedor': {
#                         'cnpj': '11258313000160',
#                         'nome': 'Devedor teste'
#                     },
#                     'valor': {
#                         'original': "0.10"
#                     },
#                     'chave': '22501210000155',
#                     'solicitacaoPagador': 'solicitacao_pagador',
#                     'infoAdicionais': [{"nome": "nome1", "valor": "valor1"}]
#         }
#         self.invalid_body = {
#                 'valor': {
#                     'original': "0.10"
#                 },
#                 'chave': '225001210000155no',
#                 'solicitacaoPagador': 'solicitacao_pagador',
#                 'infoAdicionais': [{"nome": "nome1", "valor": "valor1"}]
#         }
#         cobranca = Cob()
#         cobranca.criar_cobranca_put(self.txid_valido, self.body, url, cert, client_id, client_secret)
#
#     def test_api_criar_nova_cobranca_com_novo_txid_valido(self):
#
#         cobranca = Cob()
#         result = cobranca.criar_cobranca_put(self.txid_valido1, self.body, url, cert, client_id, client_secret)
#         self.assertEqual(result[1], 201)
#
#     def test_api_criar_nova_cobranca_com_novo_txid_valido_dados_invalidos(self):
#         cobranca = Cob()
#         result = cobranca.criar_cobranca_put(self.txid_valido, self.invalid_body, url, cert, client_id, client_secret)
#         self.assertEqual(result[1], 400)
#
#     def test_api_criar_nova_cobranca_com_txid_existente(self):
#         cobranca = Cob()
#         result = cobranca.criar_cobranca_put(self.txid_valido, self.body, url, cert, client_id, client_secret)
#         self.assertEqual(result[1], 400)
#
#     def test_api_criar_nova_cobranca_com_txid_invalido_menor_27_caracteres(self):
#         cobranca = Cob()
#         result = cobranca.criar_cobranca_put("HRAT14SSHFD", self.body, url, cert, client_id, client_secret)
#         self.assertEqual(result[1], 400)
#
#     def test_api_consultar_cobranca_com_txid_inexistente(self):
#         cobranca = Cob()
#         result = cobranca.consultar_cobranca_get(self.txid_inexistente, url, cert, client_id, client_secret)
#         self.assertEqual(result[1], 404)
#
#     def test_api_criar_nova_cobranca_com_txid_invalido_menor_27_caracteres(self):
#         cobranca = Cob()
#         result = cobranca.consultar_cobranca_get('HRAT14SSHFD', url, cert, client_id, client_secret)
#         self.assertEqual(result[1], 400)
#
#     def test_api_consultar_cobranca_com_txid_existente(self):
#         cobranca = Cob()
#         result = cobranca.consultar_cobranca_get(self.txid_valido, url, cert, client_id, client_secret)
#         self.assertEqual(result[1], 200)
#
#
#     def test_api_consultar_cobrancas_lista(self):
#         inicio = '2020-12-09T08:00:00Z'
#         fim = '2021-12-09T23:00:00Z'
#         cobranca = Cob()
#         body=None
#         r = cobranca.consultar_lista_cobranca_get(inicio, fim, body, url, cert, client_id, client_secret)
#         result = False
#         if r[1] in [200, 404]:
#             result = True
#         self.assertTrue(result)
#
#
# if __name__ == '__main__':
#     unittest.main()
